;;;; http-fs.scm


(use vfs tinyclos http-client regex-case)


(define-class <http-file-system> (<vfs:file-system>) ())

(declare (hide follow-redirects))

(define (follow-redirects url m attrs)
  (let loop ((url url) (done (list url)))
    (let-values (((ln headers i o) (http:send-request (http:make-request m url attrs))))
      (regex-case ln
	("HTTP.*\\s+30[12]\\s+.*" _
	 (close-output-port o)
	 (close-input-port i)
	 (cond ((alist-ref "location" headers string-ci=?) =>
		(lambda (loc)
		  (if (member loc done)
		      (error 'file-exists? "circular HTTP redirect" url loc)
		      (loop loc (cons loc done)))))
	       (else (error 
		      'file-exists?
		      "HTTP redirection without target location"
		      url) ) ) )
	("HTTP.*\\s+200\\s+.*" _ (values i o))
	(else (values #f #f))))))

(define-method (vfs:open-input-file (fs <http-file-system>) name modes)
  (let-values (((i o) (follow-redirects
		       name
		       'GET '(("Connection" . "close")))))
    (unless i 
      (error 'open-input-file "file not found" name) )
    (close-output-port o)
    i) )

(define-method (vfs:file-exists? (fs <http-file-system>) name)
  (let-values (((i o) (follow-redirects name 'HEAD '(("Connection" . "close")))))
    (cond (i (close-output-port o)
	     (close-input-port i)
	     #t)
	  (else #f) ) ) )

(vfs:register-file-system "http" (make <http-file-system>))
